!
! CDDL HEADER START
!
! The contents of this file are subject to the terms of the Common Development
! and Distribution License Version 1.0 (the "License").
!
! You can obtain a copy of the license at
! http://www.opensource.org/licenses/CDDL-1.0.  See the License for the
! specific language governing permissions and limitations under the License.
!
! When distributing Covered Code, include this CDDL HEADER in each file and
! include the License file in a prominent location with the name LICENSE.CDDL.
! If applicable, add the following below this CDDL HEADER, with the fields
! enclosed by brackets "[]" replaced with your own identifying information:
!
! Portions Copyright (c) [yyyy] [name of copyright owner]. All rights reserved.
!
! CDDL HEADER END
!
! Copyright (c) 2015, Regents of the University of Minnesota.
! All rights reserved.
!
! Contributors:
!    Nikhil C. Admal
!    Ellad B. Tadmor
!
! Release: This file is part of the MDStressLab git repository.
!
!e***************************************************************************
!**
!**   MODULE mod_box: ensemble properties of the simulation box
!**
!**   type definitions:
!**   ----------------
!**   type(species) :: spec(:)
!**
!**   Real Variable Definitions:
!**   -------------------------------------
!**   samplesize(:)
!**
!**   Integer Variable Definitions:
!**   ----------------------------
!**   nat    - number of atoms
!**   dimn   - dimension
!**   nsp    - number of species in the sample
!**   nsp_file-number of species in species file
!**   maxnsp - maximum number of species
!**  
!**   logical Variable Definitions:
!**   ----------------------------
!**   pbc - periodic boundary conditions
!**
!**   Contains Routines:
!**   ------------------
!**   allocate_box : allocates memory to the variable samplesize
!****************************************************************************
module mod_box
use mod_global, only : dp
use mod_types, only : species
implicit none
save
private
public allocate_box,     &
       allocate_species, &
       deallocate_box
public samplesize,       &
       samplesize_i,     &
       samplesize_f,     &
       pbc,              &
       dimn,             &
       nat,              &
       spec,             &
       nsp,              &
       max_nsp,          &
       sp_list,          &
       nsp_file

type(species), allocatable, dimension(:) :: spec
target :: spec
real(kind=dp), allocatable, dimension(:) :: samplesize,samplesize_i,samplesize_f
logical, allocatable, dimension(:) :: pbc
integer :: dimn,nat,nsp,nsp_file
integer,parameter :: max_nsp=100
character(len=2) :: sp_list(max_nsp)
contains
    !-------------------------------------------------------------------------------
    ! allocate_box -
    !
    subroutine allocate_box(cfile)
    use mod_file, only : FileExists,iofile
    use mod_output, only : string,output,nerror,error,erroroutput,heading
    implicit none
    character(len=80),intent(in) :: cfile
    integer :: ierror,iunit

    if (FileExists(cfile,.false.)) then
        call iofile(cfile,'formatted  ',iunit,.false.,.false.)
        heading=.true.
        write(string,'(a)') '**Reading configuration data**'
        call output
    else   
        nerror = 2
        error(1) = 'configuration information file not found'
        error(2) = 'in subroutine allocate_box,  Filename = ' &
                   // trim(cfile)
        call erroroutput
        stop
     endif

    read(iunit,*,iostat=ierror) dimn,nat
    write(string,2000) 'Dimension        = ',dimn; call output
    write(string,2000) 'Number of atoms  = ',nat; call output
    if (ierror .eq. 0) then
        if (dimn > 0 .and. nat > 0) then 
            allocate(samplesize(dimn))
            allocate(samplesize_i(dimn))
            allocate(samplesize_f(dimn))
            allocate(pbc(dimn))
            samplesize = 0._dp
            samplesize_i = 0._dp
            samplesize_f = 0._dp
        else
            nerror = 1
            write(error(1),2020) nat, dimn
            call erroroutput
            stop
        endif
    else
        nerror=1
        write(error(1),'(a)') 'Error reading dimn and nat'
        call erroroutput
        stop
    end if

    read(iunit,*, iostat=ierror) samplesize_i,samplesize_f
    samplesize = samplesize_f
    if (ierror .ne. 0) then
        nerror=1
        write(error(1),'(a)') 'Error reading sample size'
        call erroroutput
        stop
    endif
    write(string,2030) samplesize; call output
    read(iunit,*,iostat=ierror) pbc
    if (ierror .ne. 0) then
        nerror=1
        write(error(1),'(a)') 'Error reading periodic boundary conditions'
        call erroroutput
        stop
    endif
    write(string,2040) pbc; call output
    !-------------------------------------------------------------------------------
    ! format descriptors
    !
    2000 format(a,I10)
    2020 format('Non-positive values: nat = ',I10,', dimn = ',I2)
    2030 format('Sample size      = ',3e15.5)
    2040 format('PBC              = ',3L15)
    end subroutine allocate_box
    !-------------------------------------------------------------------------------
    ! allocate_species -
    !
    subroutine allocate_species(sfile)
    use mod_file, only : FileExists,iofile
    use mod_output, only : string,output, nerror,error,erroroutput,heading
    implicit none
    character(len=80),intent(in) :: sfile
    integer :: ierror,j=0,iunit

    if (FileExists(sfile,.false.)) then
        call iofile(sfile,'formatted  ',iunit,.false.,.false.)
        heading=.true.
        write(string,'(a)') '**Reading species data**'
        call output
    else   
        nerror = 2
        error(1) = 'species information file not found'
        error(2) = 'in subroutine allocate_species,  Filename = ' &
                   // trim(sfile)
        call erroroutput
        stop
     endif
        
    allocate(spec(max_nsp))

    write(string,'(a)') 'Species on file:'; call output
    nsp_file = 0
    do 
        j=j+1
        read(iunit,*,iostat=ierror) spec(j)%id,spec(j)%mass
        if (ierror .ne. 0) then
            exit
        endif
        nsp_file = nsp_file+1
        write(string,2010) spec(j)%id,spec(j)%mass; call output
    end do
    write(string,'(a,I5)') 'Total number of species in file = ',nsp_file; call output
    !-------------------------------------------------------------------------------
    ! Format descriptors
    !
    2010 format(2x,a2,2x,f10.5)
    end subroutine allocate_species
    !-------------------------------------------------------------------------------
    ! deallocate_box
    !
    subroutine deallocate_box
    implicit none
    deallocate( samplesize )
    deallocate( pbc )
    deallocate( spec )
    end subroutine deallocate_box
end module mod_box
